% MATLAB file given by Narimane Chatar 
% Valentin Gaudichon internship
%
% notes: 
%   - elements are called "plates" in this file
%   - TMJ = temporomandibular joints
%   - PCSA = [muscle] physiological cross-sectional area
% missing functions:
%   - readnastrandeck: reads nastran file into a structure
%   - ElementInfo: computes Element_Area, Elem_Centroid, Traction, Normal
%   - Free_edges_of_elem_group
%   - Free_edges_to_closed_paths
%
%
%
%
% variables:
%    
%  nodes: coordinates of all nodes                     size = (nbnod, 3)
%  max_number_of_muscle_groups: nb of groups                    size=(1)
%  Plate_Prop_Num: vector containing the (sorted) tags 
%                  of the groups (1, 2, 3, ...)        size = (nbgroups)
%  R_TMJ: point1 of the axis of rotation (3 coordinates)        size=(3)
%         right temporomandibular joint
%  L_TMJ: point2 of the axis of rotation (3 coordinates)        size=(3)
%         left temporomandibular joint
%  MG_Labels: labels of the groups                     size = (nbgroups)
%  Forces: force amplitudes for each group             size = (nbgroups)
%  Focal_nodes: focal point coordinates              
%               for each group                       size = (nbgroups,3)
%
%  MG_{i}_Focal_Point: shortcut to 'Focal_nodes' for group i    size=(3)
%  MG_{i}_Muscle_Force: shortcut to 'Forces' for group i        size=(1)
%
%  method: loading method
%       'uniform_traction': "ad hoc uniform traction model"
%       'gradient_traction'
%       'tangential_traction'
%       'tangential_plus_normal_traction'
%
%  muscle_group_{i}_connectivity: nodes of each element in group i
%                                                   size = (nbelem_i, 3)
%  muscle_group_{i}_element_number: element tag        size = (nbelem_i)
%  num_of_plates: nb of element for each group         size = (nbgroups)
%
%  MG_{i}_Muscle_attachment_X: x pos of the 3 nodes of each element
%                              of group i           size = (3, nbelem_i)
%  MG_{i}_Muscle_attachment_[Y|Z]: same for y,z components 
%                     (useful for "patch" display command)
%
%  MG_{i}_Element_Area                                 size = (nbelem_i)
%  MG_{i}_Elem_Centroid                             size = (nbelem_i, 3)
%  MG_{i}_Traction : tangential comp of the force   size = (nbelem_i, 3)
%  MG_{i}_Normal : normal comp of the force         size = (nbelem_i, 3)
%  MG_{i}_free_elem_edges
%  MG_{i}_outter_edge
%  MG_{i}_Outter_Edge_Nodes: list of boundardy edges     size = (nbonod)
%  MG_{i}_Outter_Edge_Nodal_Coords: coordinates       size = (nbonod, 3)
%  MG_{i}_Outter_Edge_Nodes_and_Coords: hstack of the 2 previous arrays
%  MG_{i}_TOTAL_AREA                                          size = (1)
%
%  Rem_Neg_Mom: flags (0 or 1) used for removing negative moments
%                                                      size = (nbgroups)
%
%  the script also copies the date in a structure while computing the forces
%    Muscle_Info.Muscle_attachment_X
%    Muscle_Info.Muscle_Traction
%    Muscle_Info.Muscle_Elem_Centroid
%    ...
%
%
%

close all
clear all
clc
set(0,'DefaultAxesFontSize',10);
set(0,'DefaultLineLineWidth',1.5);
set(0,'DefaultAxesFontName','Times New Roman');
screen_size = get(0,'ScreenSize');

start_time = clock;

% read NASTRAN input file (or a matlab file containing 
%   the data previously read and saved in MATLAB format)

[mesh_filename, mesh_pathname, mesh_filter_index] = uigetfile( {'*.dat','Nastran-Files (*.dat)'; ... FILETERINDEX == 1
    '*.mat','MAT-files (*.mat)'}, ... FILTERINDEX == 2
    'Choose the file with Muscle Attachment mesh','MultiSelect', 'on');
if mesh_filter_index == 1
    eval(sprintf('fe = readnastrandeck(''%s%s'')',mesh_pathname,mesh_filename))
elseif mesh_filter_index == 2
    eval(sprintf('load %s%s;',mesh_pathname,mesh_filename));
end

% read EXCEL input file containing 
%   the rotation axis ; name, focal points and forces for each group

nodes = fe.grids(:,3:5);
max_number_of_muscle_groups = max(fe.tria3s(:,2));

[muscle_load_filename, muscle_load_pathname] = uigetfile({'*.xls;*.xlsx','Excel 2003 (*.xls) or Excel 2007 (*.xlsx)'}, 'Choose Boneload Muscle Force and Focal Coordinate File (*.xls,*.xlsx)');
eval(sprintf('Plate_Prop_Num = xlsread(''%s%s'',''b6:b45'')',muscle_load_pathname,muscle_load_filename))

if issorted(Plate_Prop_Num) ~=1
    disp('Plate Property Numbers are out of order')
else
    eval(sprintf('R_TMJ = xlsread(''%s%s'',''e2:g2'')',muscle_load_pathname,muscle_load_filename))
    eval(sprintf('L_TMJ = xlsread(''%s%s'',''e3:g3'')',muscle_load_pathname,muscle_load_filename))
    eval(sprintf('[Numbers MG_Labels] = xlsread(''%s%s'',''a6:a45'')',muscle_load_pathname,muscle_load_filename))
    eval(sprintf('Forces = xlsread(''%s%s'',''c6:c45'')',muscle_load_pathname,muscle_load_filename))
    eval(sprintf('Focal_nodes = xlsread(''%s%s'',''e6:g45'')',muscle_load_pathname,muscle_load_filename))
end


for jj = 1:max_number_of_muscle_groups
    eval(sprintf('MG_%d_Focal_Point  = Focal_nodes(%d,:);',jj,jj))
    eval(sprintf('MG_%d_Muscle_Force = Forces(%d);',jj,jj))
end

%
combine_nastran_files = 'yes';
if strcmp(combine_nastran_files,'yes')==1
    eval(sprintf(' [combined_nastran_filename, combined_nastran_pathname] = uiputfile(''*.dat'',''Save Combined Nastran File to'',''%sCombined_Nastran_Mesh_and_Load.dat'')',muscle_load_pathname))
end

% Choice of Loading methods:
%   - Ad Hoc Uniform Traction        =>   method='uniform_traction'
%   - Gradient Traction              =>   method='gradient_traction'
%   - Tangential Traction            =>   method='tangential_traction'
%   - Tangential + Normal Traction   =>   method='tangential_plus_normal_traction'

h2 = figure(2);
set(gcf,'Position',[500   500   300   330]);
suptitle('Please Select Loading Method')
%title('Press A Button Below')
axis off;
uicontrol(h2,'Style','pushbutton',...
    'Position',[60,250,180,30],...
    'String','Ad Hoc Uniform Traction',...
    'Callback',' method = ''uniform_traction''; close(gcf)' );
uicontrol(h2,'Style','pushbutton',...
    'Position',[60,180,180,30],...
    'String','Gradient Traction',...
    'Callback','method = ''gradient_traction''; close(gcf)');
uicontrol(h2,'Style','pushbutton',...
    'Position',[60,110,180,30],...
    'String','Tangential Traction',...
    'Callback','method = ''tangential_traction''; close(gcf)');
uicontrol(h2,'Style','pushbutton',...
    'Position',[60,40,180,30],...
    'String','Tangential Plus Normal Traction',...
    'Callback','method = ''tangential_plus_normal_traction''; close(gcf)');
uiwait(gcf);
method
%method = 'uniform_traction'

% Fill element arrays

for jj = 1:max_number_of_muscle_groups
    eval(sprintf('disp(''Collecting Connectivity for Muscle Group #%d'') ',jj))
    I = find(fe.tria3s(:,2) == jj);
    eval(sprintf('muscle_group_%d_connectivity = fe.tria3s(I,3:5);',jj)); % Nodal Connectivity for each element of each Muscle Group Attachment
    eval(sprintf('muscle_group_%d_element_number = fe.tria3s(I,1);',jj)); % Element Number for Each Muscle Group Attachment
    eval(sprintf('num_of_plates(%d) = length(muscle_group_%d_connectivity);',jj,jj))
    clear I
end

% Fill MG_{i}_Muscle_attachment_[X|Y|Z]
%   containing the 3 x coordinates of the nodes of each element of group i

for jj = 1:1:max_number_of_muscle_groups
    figure(21)
    set(gcf,'Position',[500   500   500   100]);
    axis off;
    eval(sprintf('title(''Collecting Nodes for Muscle Group #%d'',''Fontsize'',15) ',jj))
    
    for kk = 1:num_of_plates(jj)
        eval(sprintf('MG_%d_Muscle_attachment_X(:,kk) = nodes(muscle_group_%d_connectivity(kk,:),1);',jj,jj))
        eval(sprintf('MG_%d_Muscle_attachment_Y(:,kk) = nodes(muscle_group_%d_connectivity(kk,:),2);',jj,jj))
        eval(sprintf('MG_%d_Muscle_attachment_Z(:,kk) = nodes(muscle_group_%d_connectivity(kk,:),3);',jj,jj))
    end
end



for jj = 1:1:max_number_of_muscle_groups
    eval(sprintf('[MG_%d_Element_Area, MG_%d_Elem_Centroid, MG_%d_Traction, MG_%d_Normal] = Element_info(MG_%d_Muscle_attachment_X,MG_%d_Muscle_attachment_Y,MG_%d_Muscle_attachment_Z,MG_%d_Muscle_Force,MG_%d_Focal_Point);',jj,jj,jj,jj,jj,jj,jj,jj,jj))
    % [RB] ! MG_%d_Traction => "traction" = tangential component of the force!!
    clf
    figure(21)
    set(gcf,'Position',[500   500   500   100]);
    axis off;
    eval(sprintf('title(''Calculating Free Edges for Muscle Group #%d'',''Fontsize'',15) ',jj))
    
    % The calculations below are not necessary for Uniform Traction
    if strcmp(method,'tangential_plus_normal_traction') ==1
        eval(sprintf(' MG_%d_free_elem_edges = Free_edges_of_elem_group(muscle_group_%d_connectivity);',jj,jj))
        eval(sprintf('MG_%d_outter_edge = Free_edges_to_closed_paths(nodes,MG_%d_free_elem_edges);',jj,jj))
        
        
        %%  These are the nodes of the Free Edges of Elements at the outter edge of
        %%  the Muscle attachment region
        eval(sprintf('MG_%d_Outter_Edge_Nodes = unique(MG_%d_outter_edge);',jj,jj))
        eval(sprintf('MG_%d_Outter_Edge_Nodal_Coords = [nodes(MG_%d_Outter_Edge_Nodes,:)];',jj,jj))
        eval(sprintf('MG_%d_Outter_Edge_Nodes_and_Coords = [MG_%d_Outter_Edge_Nodes  MG_%d_Outter_Edge_Nodal_Coords];',jj,jj,jj))
    end
    
    eval(sprintf('MG_%d_TOTAL_AREA = sum(MG_%d_Element_Area);',jj,jj))
    
end

% Display data:
%   (used to check the mesh and the normal orientation visually)

color_matrix = jet(max_number_of_muscle_groups);
Quiver_Vector_Plot_Scale = norm([ (max(nodes(:,1)) - min(nodes(:,1))) (max(nodes(:,2)) - min(nodes(:,2))) (max(nodes(:,3)) - min(nodes(:,3)))])/10;

figure(3)
set(gcf,'Position', [50 50 1200 900]);
grid on
hold on
% ... mesh and normals
for jj = 1:1:max_number_of_muscle_groups
    eval(sprintf('patch(MG_%d_Muscle_attachment_X,MG_%d_Muscle_attachment_Y,MG_%d_Muscle_attachment_Z,[0.25 0.25 0.35],''FaceAlpha'',0.01,''EdgeColor'',color_matrix(jj,:));',jj,jj,jj)) % - Blue
    eval(sprintf('plot3(MG_%d_Focal_Point(1),MG_%d_Focal_Point(2),MG_%d_Focal_Point(3),''sqk'',''Markersize'',10,''MarkerFaceColor'',color_matrix(jj,:))',jj,jj,jj))
    %    eval(sprintf('plot3(MG_%d_Outter_Edge_Nodal_Coords(:,1),MG_%d_Outter_Edge_Nodal_Coords(:,2),MG_%d_Outter_Edge_Nodal_Coords(:,3),''ok'',''Markersize'',7,''MarkerFaceColor'',color_matrix(jj,:))',jj,jj,jj))
    for elem_num = 1:15
        eval(sprintf('plot3(MG_%d_Elem_Centroid(%d,1),MG_%d_Elem_Centroid(%d,2),MG_%d_Elem_Centroid(%d,3),''dk'',''Markersize'',7,''MarkerFaceColor'',''k'')',jj,elem_num,jj,elem_num,jj,elem_num))
        eval(sprintf('quiver3(MG_%d_Elem_Centroid(%d,1),MG_%d_Elem_Centroid(%d,2),MG_%d_Elem_Centroid(%d,3),MG_%d_Normal(%d,1),MG_%d_Normal(%d,2),MG_%d_Normal(%d,3),Quiver_Vector_Plot_Scale,''r'',''LineWidth'',1,''Maxheadsize'',1)',jj,elem_num,jj,elem_num,jj,elem_num,jj,elem_num,jj,elem_num,jj,elem_num))
    end
end
% ... axis of rotation
if isempty(R_TMJ) ~= 1
    plot3(L_TMJ(1),L_TMJ(2),L_TMJ(3),'^k','Markersize',10,'MarkerFaceColor','k')
    plot3(R_TMJ(1),R_TMJ(2),R_TMJ(3),'^k','Markersize',10,'MarkerFaceColor','k')
    quiver3(R_TMJ(1),R_TMJ(2),R_TMJ(3),L_TMJ(1) - R_TMJ(1),L_TMJ(2) - R_TMJ(2),L_TMJ(3) - R_TMJ(3),'k','LineWidth',2,'Maxheadsize',1)
end


xlabel('X')
ylabel('Y')
zlabel('Z')

axis equal


h4 = figure(4);
set(gcf,'Position',[500   500   500   200]);
suptitle('Are the Focal Coordinates Correct?')
axis off;
uicontrol(h4,'Style','pushbutton',...
    'Position',[160,120,180,30],...
    'String','YES',...
    'Callback',' Focal_Nodes_Are_OK = ''yes''; close(gcf)' );
uicontrol(h4,'Style','pushbutton',...
    'Position',[160,50,180,30],...
    'String','NO',...
    'Callback',' Focal_Nodes_Are_OK = ''no''; close(gcf)' );
uiwait(gcf);


if strcmp(Focal_Nodes_Are_OK,'no') ==1
    exit
end

h5 = figure(5);
set(gcf,'Position',[500   500   500   200]);
suptitle('Are the normal vectors pointing OUTWARD?')
title('Normal Vectors are shown in RED')
axis off;
uicontrol(h5,'Style','pushbutton',...
    'Position',[160,120,180,30],...
    'String','YES',...
    'Callback',' Outward_Normals_Are_OK = ''yes''; close(gcf)' );
uicontrol(h5,'Style','pushbutton',...
    'Position',[160,50,180,30],...
    'String','NO',...
    'Callback',' Outward_Normals_Are_OK = ''no''; close(gcf)' );
uiwait(gcf);


Rem_Neg_Mom = zeros(length(MG_Labels),1);
if isempty(R_TMJ) ~= 1
    h6 = figure(6);
    set(gcf,'Position',[50   50   350   length(MG_Labels)*70]);
    title({'Select Muscle Groups for which ';'negative moments will be removed.';'Then close window.'},'Fontsize',15)
    axis off;
    for jj = 1:length(MG_Labels)
        eval(sprintf(' uicontrol(h6,''Style'',''radiobutton'',''Position'',[10,(length(MG_Labels) -%d)*50,180,30],''String'',MG_Labels(%d),''Callback'',''Rem_Neg_Mom(%d,:) =  1;'' )',jj,jj,jj));
    end
    uiwait(gcf);
end


eval(sprintf(' [save_filename, save_pathname] = uiputfile(''*.xls'',''Save Results Summary File to'',''%sResults_Summary.xls'')',muscle_load_pathname))

%
%  cfg = defaultParallelConfig();
%  fprintf('Using parallel configuration: %s\n',cfg);
%  matlabpool('open');


% This is the number to start using for continuation lines in the Nastran Deck
continuation_count = 400000;

for jj = 1:1:max_number_of_muscle_groups
    
    clear h_bar
    clear Moment_check
    clear Muscle_Info
    Muscle_Info.nodes = nodes;
    eval(sprintf('Muscle_Info.Muscle_attachment_X = MG_%d_Muscle_attachment_X;',jj))
    eval(sprintf('Muscle_Info.Muscle_attachment_Y = MG_%d_Muscle_attachment_Y;',jj))
    eval(sprintf('Muscle_Info.Muscle_attachment_Z = MG_%d_Muscle_attachment_Z;',jj))
    eval(sprintf('Muscle_Info.Muscle_Traction = MG_%d_Traction;',jj))
    eval(sprintf('Muscle_Info.Muscle_Elem_Centroid = MG_%d_Elem_Centroid;',jj))
    eval(sprintf('Muscle_Info.Focal_Point = MG_%d_Focal_Point;',jj))
    eval(sprintf('Muscle_Info.num_of_plates = num_of_plates(%d);',jj))
    eval(sprintf('Muscle_Info.Muscle_Force = MG_%d_Muscle_Force;',jj))
    eval(sprintf('Muscle_Info.Muscle_Elem_Area = MG_%d_Element_Area;',jj))
    if strcmp(method,'tangential_plus_normal_traction') ==1
        eval(sprintf('Muscle_Info.Muscle_Outter_Edge_Coords = MG_%d_Outter_Edge_Nodal_Coords;',jj))
        eval(sprintf('Muscle_Info.MG_Free_Element_Edges = MG_%d_free_elem_edges ;',jj))
    end
    
    % reverse normals if needed

    if strcmp(Outward_Normals_Are_OK,'no')==1
        eval(sprintf('Muscle_Info.Muscle_Normal = - MG_%d_Normal;',jj))
    elseif strcmp(Outward_Normals_Are_OK,'yes')==1
        eval(sprintf('Muscle_Info.Muscle_Normal = MG_%d_Normal;',jj))
    end
    
    NTSC = zeros(Muscle_Info.num_of_plates,1);
    avg_radius = zeros(Muscle_Info.num_of_plates,1);
    Pressure_load = zeros(Muscle_Info.num_of_plates,3);
    
    tic
    
    %    eval(sprintf('h_bar = waitbar(0,''Calculating Pressure For Muscle Group %d'')',jj));
    
    %% GLOBAL PRESSURE
    if strcmp(method,'uniform_traction') ==1
        
        for elem_num = 1:1:Muscle_Info.num_of_plates
            MG_Elem_CENT_to_FP_unit = (Muscle_Info.Focal_Point - Muscle_Info.Muscle_Elem_Centroid (elem_num,:))./norm(Muscle_Info.Focal_Point - Muscle_Info.Muscle_Elem_Centroid (elem_num,:));
            MG_Elem_Normal = Muscle_Info.Muscle_Normal(elem_num,:);
            
            % THIS CHANGES THE PRESSURE MAGNITUDE AND DIRECTION
            %
            %                                this is the "mean pressure" - same for all elements
            %                           <-----------------------------------------------------------> 
            Pressure_load(elem_num,:) = (Muscle_Info.Muscle_Force./sum(Muscle_Info.Muscle_Elem_Area))*MG_Elem_CENT_to_FP_unit;
            
            %%% Remove Negative Moments
            % REMOVE THE PRESSURE THAT CAUSES A NEGATIVE MOMENT ABOUT THE TMJ
            if Rem_Neg_Mom(jj) ==1
                Moment_check = cross(Muscle_Info.Muscle_Elem_Centroid (elem_num,:) - L_TMJ,Pressure_load(elem_num,:));
                if dot(Moment_check, [L_TMJ - R_TMJ]./norm([L_TMJ - R_TMJ]))<0
                    Pressure_load(elem_num,:) = 0*(MG_Elem_CENT_to_FP_unit);
                end
            end 
        end

        % [RB]
        % The method consists in computing a (scalar) pressure_mean = F_prescribed/total_Area 
        %    Then the "pressure_load" (3 comp vector with unit=pressure) equal
        %    pressure_load = pressure_mean*(unit vector from centroid to focal point)
        
        % [RB]: no "normalize_to_input_force" here!

    %% Gradient Traction (Pressure) Method
    elseif strcmp(method,'gradient_traction') ==1
        
        % [RB] same as above except that the force amplitude is scaled by a factor
        %      that depends on the distance of the facet to the focal point.
        %      (1 for the farthest, less for nearer elements)

        MG_Elem_CENT_to_FP_dist = zeros(Muscle_Info.num_of_plates,1);
        
        for elem_num = 1:1:Muscle_Info.num_of_plates
            MG_Elem_CENT_to_FP_dist(elem_num) = norm(Muscle_Info.Focal_Point - Muscle_Info.Muscle_Elem_Centroid (elem_num,:));
        end
        clear gradient_scale
        gradient_scale = max(MG_Elem_CENT_to_FP_dist)./MG_Elem_CENT_to_FP_dist;
%        gradient_scale = (MG_Elem_CENT_to_FP_dist)./(max(MG_Elem_CENT_to_FP_dist) - mean(MG_Elem_CENT_to_FP_dist));
        eval(sprintf('MG_%d_Elem_pressure_gradient_scale = gradient_scale;',jj))
        for elem_num = 1:1:Muscle_Info.num_of_plates
        
            MG_Elem_CENT_to_FP_unit = (Muscle_Info.Focal_Point - Muscle_Info.Muscle_Elem_Centroid (elem_num,:))./norm(Muscle_Info.Focal_Point - Muscle_Info.Muscle_Elem_Centroid (elem_num,:));
            MG_Elem_Normal = Muscle_Info.Muscle_Normal(elem_num,:);
            
            % THIS CHANGES THE PRESSURE MAGNITUDE AND DIRECTION
            Pressure_load(elem_num,:) = gradient_scale(elem_num).*(Muscle_Info.Muscle_Force./sum(Muscle_Info.Muscle_Elem_Area))*MG_Elem_CENT_to_FP_unit;           
        end

        normalize_to_input_force = Muscle_Info.Muscle_Force./sum(sqrt(Pressure_load(:,1).^2+Pressure_load(:,2).^2+Pressure_load(:,3).^2).*(Muscle_Info.Muscle_Elem_Area(:)));

        for elem_num = 1:1:Muscle_Info.num_of_plates
        %%% Remove Negative Moments
        % REMOVE THE PRESSURE THAT CAUSES A NEGATIVE MOMENT ABOUT THE TMJ
            if Rem_Neg_Mom(jj) ==1
                Moment_check = cross(Muscle_Info.Muscle_Elem_Centroid (elem_num,:) - L_TMJ,Pressure_load(elem_num,:));
                if dot(Moment_check, [L_TMJ - R_TMJ]./norm([L_TMJ - R_TMJ]))<0
                    Pressure_load(elem_num,:) = 0*(MG_Elem_CENT_to_FP_unit);
                end
            end
        end
            
            
        %% TANGENTIAL ONLY
    elseif strcmp(method,'tangential_traction') ==1
        
        for elem_num = 1:1:Muscle_Info.num_of_plates
            
            MG_Elem_CENT_to_FP_unit = (Muscle_Info.Focal_Point - Muscle_Info.Muscle_Elem_Centroid (elem_num,:))./norm(Muscle_Info.Focal_Point - Muscle_Info.Muscle_Elem_Centroid (elem_num,:));
            MG_Elem_Normal = Muscle_Info.Muscle_Normal(elem_num,:);
            % NOTE MG_Elem_Normal is already a unit vector
            
            if dot(MG_Elem_CENT_to_FP_unit,MG_Elem_Normal) > 0
                Pressure_load(elem_num,:) = (Muscle_Info.Muscle_Force./sum(Muscle_Info.Muscle_Elem_Area))*MG_Elem_CENT_to_FP_unit;
            else
                % tv = "tangent vector"
                tv = Muscle_Info.Muscle_Traction(elem_num,:)./(norm(Muscle_Info.Muscle_Traction(elem_num,:)));
                Pressure_load(elem_num,:) = (Muscle_Info.Muscle_Force./sum(Muscle_Info.Muscle_Elem_Area))*(tv);
            end
            
            %%% Remove Negative Moments
            % REMOVE THE PRESSURE THAT CAUSES A NEGATIVE MOMENT ABOUT THE TMJ
            if Rem_Neg_Mom(jj) ==1
                Moment_check = cross(Muscle_Info.Muscle_Elem_Centroid (elem_num,:) - L_TMJ,Pressure_load(elem_num,:));
                if dot(Moment_check, [L_TMJ - R_TMJ]./norm([L_TMJ - R_TMJ]))<0
                    Pressure_load(elem_num,:) = 0*(MG_Elem_CENT_to_FP_unit);
                end
            end
        end

        % [RB]: no "normalize_to_input_force" here!
        
    elseif strcmp(method,'tangential_plus_normal_traction') ==1
        
        %% TANGENTIAL AND NORMAL TRACTION (WITH NEG CURV REMOVED)
        
        for elem_num = 1:1:Muscle_Info.num_of_plates
            
            MG_Elem_CENT_to_FP_unit = (Muscle_Info.Focal_Point - Muscle_Info.Muscle_Elem_Centroid (elem_num,:))./norm(Muscle_Info.Focal_Point - Muscle_Info.Muscle_Elem_Centroid (elem_num,:));
            MG_Elem_Normal = Muscle_Info.Muscle_Normal(elem_num,:);
            
            % NOTE MG_Elem_Normal is already a unit vector
            
            if dot(MG_Elem_CENT_to_FP_unit,MG_Elem_Normal) > 0
                
                %if acos(dot(MG_Elem_CENT_to_FP_unit,MG_Elem_Normal))*180/pi < 90;
                %    If the angle between the vector from the centroid of the element to the
                %    focal point and the element  outward normal is less than 90 degress then the
                %    pressure will be applied to the face of the element in the direction
                %    of the focal node
                Pressure_load(elem_num,:) = (Muscle_Info.Muscle_Force./sum(Muscle_Info.Muscle_Elem_Area))*MG_Elem_CENT_to_FP_unit;
                
            else
                
                Norm_Press_info = New_Path_Calculation_Subroutine(Muscle_Info,elem_num);
                NTSC(elem_num) = Norm_Press_info.normal_traction_scale_factor;
                avg_radius(elem_num) = Norm_Press_info.Radius_avg;
                %    If the angle between the vector from the centroid of the element to the
                %    focal point and the element outward normal is 90 degress or more then the
                %    pressure will be applied to the face of the element in both in the plane
                %    of the element in direction of the focal node and normal to the element.
                %    We use this formula:
                %    Tt (tv1 i + tv2 j + tv3 k) + Tn(nv1 i + nv2 j + nv3 k);
                %    where:
                %    t1,t2,t3 indicate the direction of the inplane traction vector ([RB]: "traction" means "tangent" here)
                %    n1,n2,n3 indicate the direction of the outward normal vector
                %    Tt = The Tangential Traction (F/A)
                %    Tn = Tt*(s(r)/R(r))
                %
                %    See Grosse et al 2007 - Techniques for modeling muscle induced forces
                %    in FEMs of Skeletal Structures, The Anatomical Record, v.290, pp
                %    1069-1088
                
                % FORCE VECTOR DIRECTIONS
                % tangent vector
                tv = Muscle_Info.Muscle_Traction(elem_num,:)./(norm(Muscle_Info.Muscle_Traction(elem_num,:)));
                % normal vector
                nv = - Muscle_Info.Muscle_Normal(elem_num,:);
                
                % THIS CHANGES THE PRESSURE MAGNITUDE AND DIRECTION
                Pressure_load(elem_num,:) = (Muscle_Info.Muscle_Force./sum(Muscle_Info.Muscle_Elem_Area))*(tv + NTSC(elem_num)*nv);
                
                %%% Remove Negative Moments
                % REMOVE THE PRESSURE THAT CAUSES A NEGATIVE MOMENT ABOUT THE TMJ
                if Rem_Neg_Mom(jj) ==1
                    Moment_check = cross(Muscle_Info.Muscle_Elem_Centroid (elem_num,:) - L_TMJ,Pressure_load(elem_num,:));
                    if dot(Moment_check, [L_TMJ - R_TMJ]./norm([L_TMJ - R_TMJ]))<0
                        Pressure_load(elem_num,:) = 0*(MG_Elem_CENT_to_FP_unit);
                    end
                end
            end
            
        end
        
        
    end
    
    
    toc
    eval(sprintf('MG_%d_NTSC = NTSC;',jj))
    
    if strcmp(method,'gradient_traction') ==1
        eval(sprintf('MG_%d_Pressure_load = normalize_to_input_force.*Pressure_load;',jj))
    else
        eval(sprintf('MG_%d_Pressure_load = Pressure_load;',jj))
    end
    
    
    
    figure(21)
    set(gcf,'Position',[500   500   500   100]);
    axis off;
    eval(sprintf('title(''Writing Pressures from Muscle Group %d to Loading File'',''Fontsize'',15) ',jj))
    
    %% WRITE LOADING FILE TO LOCATION CHOSEN AT TOP OF PROGRAM
    pressure_load_pathname = muscle_load_pathname ;
    eval(sprintf('pressure_load_filename = ''Loaded_Nastran_PLOAD4_ALL_MGs_%d_%d_%d_%d_%d.nas'';',start_time(2),start_time(3),start_time(1),start_time(4),start_time(5)))
    eval(sprintf('fid_pressure = fopen(''%s%s_%s'', ''a'')',pressure_load_pathname,method,pressure_load_filename));
    
    %    eval(sprintf('fid = fopen(''%s%s_%s'', ''a'')',pressure_load_pathname,method,'DOESTHISWORK'))
    
    %%% PRINT HEADER FOR MUSCLE LOADING FILE
    fprintf(fid_pressure,'$  SURFACE AND SOLID ELEMENT LOADS\n');
    fprintf(fid_pressure,'$--------------------------------------------------------------------------------\n');
    fprintf(fid_pressure,'$\n');
    fprintf(fid_pressure,'$\n');
    
    %% ENTER INFO INTO THE PLOAD SCRIPT
    SID = 1 ; % Load Set ID number
    %EID = 1236 ; % Element ID number
    %P1  = 400e5 % norm(Pressure_load); % Pressure at Node 1
    %P2  % Pressure at Node 2
    %P3  % Pressure at Node 3
    %P4  % Pressure at Node 4
    CID = 0    ; %Coordininate System ID number
    %N1 = 0.5   ; %Pressure_load_unit_vec(1) % n1 Direction of Pressure vector written in the frame of the coordinate system identified by CID
    %N2 = 0.5   ; %Pressure_load_unit_vec(2) % n2 Direction of Pressure vector written in the frame of the coordinate system identified by CID
    %N3 = 0.25  ; %Pressure_load_unit_vec(3) % n3 Direction of Pressure vector written in the frame of the coordinate system identified by CID
    
    clear Pressure_load_mag Pressure_load_unit_vec
    
    for mm = 1:1:num_of_plates(jj)
        eval(sprintf('muscle_group_element_number = muscle_group_%d_element_number;',jj))
        eval(sprintf('Pressure_load_mag = norm(MG_%d_Pressure_load(mm,:));',jj))
        eval(sprintf('Pressure_load_unit_vec = MG_%d_Pressure_load(mm,:)./Pressure_load_mag;',jj))
        
        %                              % write_nastran_pload(fid,SID,EID,P1,CID,N1,N2,N3,continuation_number)
        continuation_count_end = write_nastran_pload_ZJT(fid_pressure,SID,muscle_group_element_number(mm),Pressure_load_mag,CID,...
            Pressure_load_unit_vec(1,1),Pressure_load_unit_vec(1,2),Pressure_load_unit_vec(1,3),continuation_count);
        
        continuation_count = continuation_count_end+1;
        
    end
    
    fclose(fid_pressure);
    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure(21)
set(gcf,'Position',[500   500   500   100]);
axis off;
title('Calculating Values for Results Summary file','Fontsize',15)
%% CALCULATE OUTPUT RESULTS - MOMENTS AND MOMENT ARMS
for jj = 1:max_number_of_muscle_groups
    
    for elem_num = 1:num_of_plates(jj)
        eval(sprintf('MG_%d_Force_Vector(elem_num,:) = MG_%d_Element_Area(elem_num)*MG_%d_Pressure_load(elem_num,:);',jj,jj,jj))
        eval(sprintf('Total_MG_%d_Force_Mag(elem_num) = norm(MG_%d_Force_Vector(elem_num,:));',jj,jj))
        if isempty(R_TMJ) ~= 1
            eval(sprintf('MG_%d_L_TMJ_Moment(elem_num,:) = cross(MG_%d_Elem_Centroid(elem_num,:) - L_TMJ,MG_%d_Pressure_load(elem_num,:)*MG_%d_Element_Area(elem_num));',jj,jj,jj,jj))
            eval(sprintf('MG_%d_R_TMJ_Moment(elem_num,:) = cross(MG_%d_Elem_Centroid(elem_num,:) - R_TMJ,MG_%d_Pressure_load(elem_num,:)*MG_%d_Element_Area(elem_num));',jj,jj,jj,jj))
        end
        
    end
    
    
    eval(sprintf('MuscleForce = MG_%d_Muscle_Force;',jj))
    eval(sprintf('SumForce = sum(Total_MG_%d_Force_Mag);',jj))
    %eval(sprintf('disp(''Muscle Force = %d  . . .  Summed Force from Pressure = %d '')',MuscleForce,SumForce))
    %    eval(sprintf('Total_MG_%d_Force_Vector = [sum(MG_%d_Force_Vector(:,1)) sum(MG_%d_Force_Vector(:,2)) sum(MG_%d_Force_Vector(:,3))];',jj,jj,jj,jj))
    eval(sprintf('Total_MG_%d_Force_Vector = sum(MG_%d_Force_Vector);',jj,jj))
    if isempty(R_TMJ) ~= 1
        % ------------------------ Left TMJ Calculations
        eval(sprintf('MG_%d_Sum_L_TMJ_Moments = sum(MG_%d_L_TMJ_Moment);',jj,jj))
        eval(sprintf('MG_%d_L_TMJ_rx = mean(unique(MG_%d_Muscle_attachment_X));',jj,jj));
        eval(sprintf('MG_%d_L_TMJ_ry = (MG_%d_Sum_L_TMJ_Moments(3) - Total_MG_%d_Force_Vector(2)*MG_%d_L_TMJ_rx)/ (-Total_MG_%d_Force_Vector(1));',jj,jj,jj,jj,jj))
        eval(sprintf('MG_%d_L_TMJ_rz = (MG_%d_Sum_L_TMJ_Moments(2) + Total_MG_%d_Force_Vector(3)*MG_%d_L_TMJ_rx)/ ( Total_MG_%d_Force_Vector(1));',jj,jj,jj,jj,jj))
        eval(sprintf('MG_%d_Force_centroid_L_TMJ = L_TMJ + [MG_%d_L_TMJ_rx MG_%d_L_TMJ_ry MG_%d_L_TMJ_rz];',jj,jj,jj,jj))
        eval(sprintf('MG_%d_L_TMJ_line_of_action = [MG_%d_L_TMJ_rx + 0.0001  (MG_%d_Sum_L_TMJ_Moments(3) - Total_MG_%d_Force_Vector(2)*(MG_%d_L_TMJ_rx+0.0001))/ (-Total_MG_%d_Force_Vector(1)) (MG_%d_Sum_L_TMJ_Moments(2) + Total_MG_%d_Force_Vector(3)*(MG_%d_L_TMJ_rx+0.0001))/ ( Total_MG_%d_Force_Vector(1))] - [MG_%d_L_TMJ_rx MG_%d_L_TMJ_ry MG_%d_L_TMJ_rz];',jj,jj,jj,jj,jj,jj,jj,jj,jj,jj,jj,jj,jj))
        eval(sprintf('MG_%d_perp_dist_to_focal_node_L_TMJ = norm(cross(Total_MG_%d_Force_Vector,MG_%d_Focal_Point- MG_%d_Force_centroid_L_TMJ))./(norm(Total_MG_%d_Force_Vector));',jj,jj,jj,jj,jj))
        
        % ------------------------ Right TMJ Calculations
        eval(sprintf('MG_%d_Sum_R_TMJ_Moments = sum(MG_%d_R_TMJ_Moment);',jj,jj))
        eval(sprintf('MG_%d_R_TMJ_rx = mean(unique(MG_%d_Muscle_attachment_X));',jj,jj));
        eval(sprintf('MG_%d_R_TMJ_ry = (MG_%d_Sum_R_TMJ_Moments(3) - Total_MG_%d_Force_Vector(2)*MG_%d_R_TMJ_rx)/ (-Total_MG_%d_Force_Vector(1));',jj,jj,jj,jj,jj))
        eval(sprintf('MG_%d_R_TMJ_rz = (MG_%d_Sum_R_TMJ_Moments(2) + Total_MG_%d_Force_Vector(3)*MG_%d_R_TMJ_rx)/ ( Total_MG_%d_Force_Vector(1));',jj,jj,jj,jj,jj))
        eval(sprintf('MG_%d_Force_centroid_R_TMJ = R_TMJ + [MG_%d_R_TMJ_rx MG_%d_R_TMJ_ry MG_%d_R_TMJ_rz];',jj,jj,jj,jj))
        eval(sprintf('MG_%d_R_TMJ_line_of_action = [MG_%d_R_TMJ_rx + 0.0001  (MG_%d_Sum_R_TMJ_Moments(3) - Total_MG_%d_Force_Vector(2)*(MG_%d_R_TMJ_rx+0.0001))/ (-Total_MG_%d_Force_Vector(1)) (MG_%d_Sum_R_TMJ_Moments(2) + Total_MG_%d_Force_Vector(3)*(MG_%d_R_TMJ_rx+0.0001))/ ( Total_MG_%d_Force_Vector(1))] - [MG_%d_R_TMJ_rx MG_%d_R_TMJ_ry MG_%d_R_TMJ_rz];',jj,jj,jj,jj,jj,jj,jj,jj,jj,jj,jj,jj,jj))
        eval(sprintf('MG_%d_perp_dist_to_focal_node_R_TMJ = norm(cross(Total_MG_%d_Force_Vector,MG_%d_Focal_Point- MG_%d_Force_centroid_R_TMJ))./(norm(Total_MG_%d_Force_Vector));',jj,jj,jj,jj,jj))
        
        %     eval(sprintf('MG_%d_moment_arm = norm(cross(R_TMJ-L_TMJ,Total_MG_%d_Force_Vector))./(norm(Total_MG_%d_Force_Vector));',jj,jj,jj))
        eval(sprintf('MG_%d_moment_arm_R_TMJ = abs(dot((MG_%d_Force_centroid_R_TMJ - R_TMJ),cross(L_TMJ-R_TMJ,Total_MG_%d_Force_Vector))./(norm(cross(L_TMJ-R_TMJ,Total_MG_%d_Force_Vector))));',jj,jj,jj,jj))
        eval(sprintf('MG_%d_moment_arm_L_TMJ = abs(dot((MG_%d_Force_centroid_L_TMJ - L_TMJ),cross(L_TMJ-R_TMJ,Total_MG_%d_Force_Vector))./(norm(cross(L_TMJ-R_TMJ,Total_MG_%d_Force_Vector))));',jj,jj,jj,jj))
        
        % Total moment about TMJ Axis -
        %             disp('TOTAL MOMENT ABOUT TMJ AXIS');
        %             eval(sprintf('dot(MG_%d_Sum_L_TMJ_Moments, [L_TMJ - R_TMJ]./norm([L_TMJ - R_TMJ]));',jj))
        %             eval(sprintf('dot(MG_%d_Sum_R_TMJ_Moments, [L_TMJ - R_TMJ]./norm([L_TMJ - R_TMJ]));',jj))
    end
end


if isempty(R_TMJ) ~= 1
    
    results(1,:) = {'MG', 'Input Muscle Force','Applied Muscle Force Mag','Vector Force Mag', 'Perp Dist. Error', 'Moment Arm','Moment about TMJ Axis'};
    for jj = 1:1:max_number_of_muscle_groups
        eval(sprintf('results(%d+2,:) = { %d, MG_%d_Muscle_Force, sum(Total_MG_%d_Force_Mag), norm(Total_MG_%d_Force_Vector), MG_%d_perp_dist_to_focal_node_L_TMJ,  MG_%d_moment_arm_L_TMJ, dot(MG_%d_Sum_L_TMJ_Moments, [L_TMJ - R_TMJ]./norm([L_TMJ - R_TMJ]))};',jj,jj,jj,jj,jj,jj,jj,jj))
    end
    eval(sprintf('xlswrite(''%s%s'',results,''RESULTS_SUMMARY'',''A1'');',save_pathname,save_filename))
    
else
    
    results(1,:) = {'MG', 'Input Muscle Force','Applied Muscle Force Mag','Vector Force Mag'};
    for jj = 1:1:max_number_of_muscle_groups
        eval(sprintf('results(%d+2,:) = { %d, MG_%d_Muscle_Force, sum(Total_MG_%d_Force_Mag), norm(Total_MG_%d_Force_Vector)};',jj,jj,jj,jj,jj))
    end
    eval(sprintf('xlswrite(''%s%s'',results,''RESULTS_SUMMARY'',''A1'');',mesh_pathname,'Results_Summary'))
    
end


%% CLOSE PARALLEL PROCESSING
%matlabpool('close')

%% COMBINE NASTRAN FILES

if strcmp(combine_nastran_files,'yes')==1
    figure(21)
    set(gcf,'Position',[500   500   500   100]);
    axis off;
    title({'Combining Nastran Mesh and Loading Files ';'This should just take a minute or two'},'Fontsize',15)
    
    
    tic
    eval(sprintf('fid_geom = fopen(''%s%s'',''r'')',mesh_pathname,mesh_filename));
    eval(sprintf('fid_loaded_geom = fopen(''%s%s'',''w'')' ,combined_nastran_pathname,combined_nastran_filename));
    eval(sprintf('fid_load = fopen(''%s%s_%s'')',pressure_load_pathname,method,pressure_load_filename));
    
    Nastran_line = ' ' ;% Need to setup Nastan_line Varaible for "ENDDATA" comparison
    while strcmp(Nastran_line,'ENDDATA')~=1
        Nastran_line = fgetl(fid_geom);
        
        % START WRITING LOADING INFORMATION BEFORE "SURFACE ELEMENT PROPERTIES"
        
        if strcmp(Nastran_line,'$  SURFACE ELEMENT PROPERTIES')==1
            Loading_file = fgetl(fid_load);
            while Loading_file~=-1
                fprintf(fid_loaded_geom,'%s\n',Loading_file);
                Loading_file = fgetl(fid_load);
            end
        end
        fprintf(fid_loaded_geom,'%s\n',Nastran_line);
    end
    fclose('all');
    toc
end


%sprintf('TOTAL RUN TIME =  %d Hours %d Minutes %2.0d Seconds',floor(etime(clock,start_time)/3600),floor(rem(etime(clock,start_time),3600)/60),floor(rem(etime(clock,start_time),60)))
time = etime2(start_time,clock);

days = time(1);%floor(time/(24*3600));
hours = time(2);%floor(time/3600)- ()*floor(time/(24*3600));
minutes = time(3);%floor(time/60) - floor(time/3600);
seconds = time(4);%time - 60*floor(time/60) - 3600*floor(time/3600);

eval(sprintf('dlmwrite(''%s\\Time.txt'',[days hours minutes seconds ])',muscle_load_pathname))

eval(sprintf('delete(''%s%s_%s'')',pressure_load_pathname,method,pressure_load_filename));


% %% Plotting Stuff
%
close all
figure(7)
set(gcf,'Position', [50 50 1200 900]);
grid on
hold on
for jj = 1:1:max_number_of_muscle_groups
    eval(sprintf('patch(MG_%d_Muscle_attachment_X,MG_%d_Muscle_attachment_Y,MG_%d_Muscle_attachment_Z,[0.25 0.25 0.35],''FaceAlpha'',0.01,''EdgeColor'',color_matrix(jj,:));',jj,jj,jj)) % - Blue
    eval(sprintf('plot3(MG_%d_Focal_Point(1),MG_%d_Focal_Point(2),MG_%d_Focal_Point(3),''sqk'',''Markersize'',10,''MarkerFaceColor'',color_matrix(jj,:))',jj,jj,jj))
    %    eval(sprintf('plot3(MG_%d_Outter_Edge_Nodal_Coords(:,1),MG_%d_Outter_Edge_Nodal_Coords(:,2),MG_%d_Outter_Edge_Nodal_Coords(:,3),''ok'',''Markersize'',7,''MarkerFaceColor'',color_matrix(jj,:))',jj,jj,jj))
    eval(sprintf('plot3(MG_%d_Focal_Point(1),MG_%d_Focal_Point(2),MG_%d_Focal_Point(3),''sqk'',''Markersize'',10,''MarkerFaceColor'',color_matrix(jj,:))',jj,jj,jj))
    
    if isempty(R_TMJ) ~= 1
        eval(sprintf('FV_dir = dot( (MG_%d_Force_centroid_L_TMJ-MG_%d_Focal_Point)./norm(MG_%d_Force_centroid_L_TMJ-MG_%d_Focal_Point),Total_MG_%d_Force_Vector./norm(Total_MG_%d_Force_Vector));',jj,jj,jj,jj,jj,jj))
        
        if FV_dir<0
            eval(sprintf('quiver3(MG_%d_Force_centroid_L_TMJ(1),MG_%d_Force_centroid_L_TMJ(2),MG_%d_Force_centroid_L_TMJ(3),Total_MG_%d_Force_Vector(1)./norm(Total_MG_%d_Force_Vector),Total_MG_%d_Force_Vector(2)./norm(Total_MG_%d_Force_Vector),Total_MG_%d_Force_Vector(3)./norm(Total_MG_%d_Force_Vector),Quiver_Vector_Plot_Scale,''Color'',color_matrix(jj,:),''LineWidth'',2,''Maxheadsize'',2)',jj,jj,jj,jj,jj,jj,jj,jj,jj))
            eval(sprintf('plot3(MG_%d_Force_centroid_L_TMJ(1),MG_%d_Force_centroid_L_TMJ(2),MG_%d_Force_centroid_L_TMJ(3),''hk'',''Markersize'',15,''MarkerFaceColor'',color_matrix(jj,:))',jj,jj,jj))
            %        eval(sprintf('line([MG_%d_Force_centroid_L_TMJ(1) MG_%d_Focal_Point(1)] , [MG_%d_Force_centroid_L_TMJ(2) MG_%d_Focal_Point(2)] , [MG_%d_Force_centroid_L_TMJ(3)  MG_%d_Focal_Point(3)],''Color'',color_matrix(jj,:))',jj,jj,jj,jj,jj,jj))
            %        eval(sprintf('line([MG_%d_Force_centroid_R_TMJ(1) MG_%d_Focal_Point(1)] , [MG_%d_Force_centroid_R_TMJ(2) MG_%d_Focal_Point(2)] , [MG_%d_Force_centroid_R_TMJ(3)  MG_%d_Focal_Point(3)],''Color'',color_matrix(jj,:))',jj,jj,jj,jj,jj,jj))
        else
            eval(sprintf('quiver3(MG_%d_Force_centroid_R_TMJ(1),MG_%d_Force_centroid_R_TMJ(2),MG_%d_Force_centroid_R_TMJ(3),Total_MG_%d_Force_Vector(1)./norm(Total_MG_%d_Force_Vector),Total_MG_%d_Force_Vector(2)./norm(Total_MG_%d_Force_Vector),Total_MG_%d_Force_Vector(3)./norm(Total_MG_%d_Force_Vector),Quiver_Vector_Plot_Scale,''Color'',color_matrix(jj,:),''LineWidth'',2,''Maxheadsize'',2)',jj,jj,jj,jj,jj,jj,jj,jj,jj))
            eval(sprintf('plot3(MG_%d_Force_centroid_R_TMJ(1),MG_%d_Force_centroid_R_TMJ(2),MG_%d_Force_centroid_R_TMJ(3),''hk'',''Markersize'',15,''MarkerFaceColor'',color_matrix(jj,:))',jj,jj,jj))
            
        end
    end
    
    
end
if isempty(R_TMJ) ~= 1
    plot3(L_TMJ(1),L_TMJ(2),L_TMJ(3),'^k','Markersize',10,'MarkerFaceColor','k')
    plot3(R_TMJ(1),R_TMJ(2),R_TMJ(3),'^k','Markersize',10,'MarkerFaceColor','k')
    quiver3(R_TMJ(1),R_TMJ(2),R_TMJ(3),L_TMJ(1) - R_TMJ(1),L_TMJ(2) - R_TMJ(2),L_TMJ(3) - R_TMJ(3),'k','LineWidth',2,'Maxheadsize',1)
    suptitle('Muscle Attachment Areas & Total Force Vectors')
else
    suptitle('Muscle Attachment Areas')
end

xlabel('X')
ylabel('Y')
zlabel('Z')
axis equal


